@@ -1,5 +1,5 @@
 from functools import partial
-from collections import defaultdict
+from collections import defaultdict, deque
 from random import random
 
 from selenium.common.exceptions import NoSuchElementException
@@ -7,12 +7,6 @@ from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.common.by import By
 
 from .hv import HVDriver, searchxpath_fun
-from .hv_battle_stat_provider import (
-    StatProviderHP,
-    StatProviderMP,
-    StatProviderSP,
-    StatProviderOvercharge,
-)
 from .hv_battle_ponychart import PonyChart
 from .hv_battle_item_provider import ItemProvider
 from .hv_battle_action_manager import ElementActionManager
@@ -20,26 +14,20 @@ from .hv_battle_skill_manager import SkillManager
 from .hv_battle_buff_manager import BuffManager
 from .hv_battle_monster_status_manager import MonsterStatusManager
 from .hv_battle_log import LogProvider
+from .hv_battle_dashboard import BattleDashBoard
 from .pause_controller import PauseController
 
-
-def interleave_even_odd(nums):
-    if 0 in nums:
-        nums = sorted(nums[:-1]) + [0]  # 0在最後
-    else:
-        nums = sorted(nums)
-    even = nums[::2]
-    odd = nums[1::2]
-    result = []
-    i = j = 0
-    for k in range(len(nums)):
-        if k % 2 == 0 and i < len(even):
-            result.append(even[i])
-            i += 1
-        elif j < len(odd):
-            result.append(odd[j])
-            j += 1
-    return result
+MONSTER_DEBUFF_TO_CHARACTER_SKILL = {
+    "Imperiled": "Imperil",
+    "Weakened": "Weaken",
+    "Slowed": "Slow",
+    "Asleep": "Sleep",
+    "Confused": "Confuse",
+    "Magically Snared": "MagNet",
+    "Blinded": "Blind",
+    "Vital Theft": "Drain",
+    "Silenced": "Silence",
+}
 
 
 def return_false_on_nosuch(fun):
@@ -86,14 +74,29 @@ class StatThreshold:
 class BattleDriver(HVDriver):
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
+
+        self.battle_dashboard = BattleDashBoard(self)
+        self.element_action_manager = ElementActionManager(self, self.battle_dashboard)
+
         self.with_ofc = "isekai" not in self.driver.current_url
-        self._logprovider = LogProvider(self)
-        self._itemprovider = ItemProvider(self)
-        self._skillmanager = SkillManager(self)
-        self._buffmanager = BuffManager(self)
-        self._monsterstatusmanager = MonsterStatusManager(self)
+        self._logprovider = LogProvider(self, self.battle_dashboard)
+        self._itemprovider = ItemProvider(self, self.battle_dashboard)
+        self._skillmanager = SkillManager(self, self.battle_dashboard)
+        self._buffmanager = BuffManager(self, self.battle_dashboard)
+        self._monsterstatusmanager = MonsterStatusManager(self, self.battle_dashboard)
         self.pausecontroller = PauseController()
         self.turn = -1
+        self.round = -1
+        self.pround = -1
+
+    def clear_cache(self) -> None:
+        # 重新解析戰鬥儀表板以獲取最新的怪物狀態
+        self.battle_dashboard.refresh()
+        self._monsterstatusmanager.clear_cache()
+        self.round = self._logprovider.current_round
+
+    def reset_pround(self) -> None:
+        self.pround = self.round
 
     def set_battle_parameters(
         self, statthreshold: StatThreshold, forbidden_skills: list[list]
@@ -109,13 +112,13 @@ class BattleDriver(HVDriver):
     def get_stat_percent(self, stat: str) -> float:
         match stat.lower():
             case "hp":
-                value = StatProviderHP(self).get_percent()
+                value = self.battle_dashboard.character.vitals.hp
             case "mp":
-                value = StatProviderMP(self).get_percent()
+                value = self.battle_dashboard.character.vitals.mp
             case "sp":
-                value = StatProviderSP(self).get_percent()
+                value = self.battle_dashboard.character.vitals.sp
             case "overcharge":
-                value = StatProviderOvercharge(self).get_percent()
+                value = self.battle_dashboard.character.vitals.overcharge
             case _:
                 raise ValueError(f"Unknown stat: {stat}")
         return value
@@ -128,10 +131,6 @@ class BattleDriver(HVDriver):
         round_str = f"Round {self._logprovider.current_round:>3} / {self._logprovider.total_round:<3}"
         return [f"{turn_str} {round_str} {line}" for line in new_logs]
 
-    @property
-    def is_with_spirit_stance(self) -> bool:
-        return StatProviderOvercharge(self).get_spirit_stance_status() == "activated"
-
     def use_item(self, key: str) -> bool:
         return self._itemprovider.use(key)
 
@@ -156,102 +155,82 @@ class BattleDriver(HVDriver):
                         return False
         return apply_buff()
 
-    @property
-    def monster_alive_count(self) -> int:
-        return self._monsterstatusmanager.alive_count
-
-    @property
-    def monster_alive_ids(self) -> list[int]:
-        return self._monsterstatusmanager.alive_monster_ids
-
     @property
     def system_monster_alive_ids(self) -> list[int]:
         return self._monsterstatusmanager.alive_system_monster_ids
 
-    def get_monster_id_by_name(self, name: str) -> int:
-        """
-        根據怪物名稱取得對應的 monster id（如 mkey_0 會回傳 0）。
-        """
-        return self._monsterstatusmanager.get_monster_id_by_name(name)
-
     @return_false_on_nosuch
     def check_hp(self) -> bool:
         if self.get_stat_percent("hp") < self.statthreshold.hp[0]:
-            if any(
-                [
-                    self.use_item("Health Gem"),
-                    self.click_skill("Full-Cure"),
-                    self.use_item("Health Potion"),
-                    self.use_item("Health Elixir"),
-                    self.use_item("Last Elixir"),
-                    self.click_skill("Cure"),
-                ]
-            ):
-                return True
+            for fun in [
+                partial(self.use_item, "Health Gem"),
+                partial(self.click_skill, "Full-Cure"),
+                partial(self.use_item, "Health Potion"),
+                partial(self.use_item, "Health Elixir"),
+                partial(self.use_item, "Last Elixir"),
+                partial(self.click_skill, "Cure"),
+            ]:
+                if fun():
+                    return True
 
         if self.get_stat_percent("hp") < self.statthreshold.hp[1]:
-            if any(
-                [
-                    self.use_item("Health Gem"),
-                    self.click_skill("Cure"),
-                    self.use_item("Health Potion"),
-                ]
-            ):
-                return True
+            for fun in [
+                partial(self.use_item, "Health Gem"),
+                partial(self.click_skill, "Cure"),
+                partial(self.use_item, "Health Potion"),
+            ]:
+                if fun():
+                    return True
 
         return False
 
     @return_false_on_nosuch
     def check_mp(self) -> bool:
         if self.get_stat_percent("mp") < self.statthreshold.mp[0]:
-            if any(
-                [
-                    self.use_item("Mana Gem"),
-                    self.use_item("Mana Potion"),
-                    self.use_item("Mana Elixir"),
-                    self.use_item("Last Elixir"),
-                ]
-            ):
-                return True
+            for fun in [
+                partial(self.use_item, "Mana Gem"),
+                partial(self.use_item, "Mana Potion"),
+                partial(self.use_item, "Mana Elixir"),
+                partial(self.use_item, "Last Elixir"),
+            ]:
+                if fun():
+                    return True
 
         if self.get_stat_percent("mp") < self.statthreshold.mp[1]:
-            if any(
-                [
-                    self.use_item("Mana Gem"),
-                    self.use_item("Mana Potion"),
-                ]
-            ):
-                return True
+            for fun in [
+                partial(self.use_item, "Mana Gem"),
+                partial(self.use_item, "Mana Potion"),
+            ]:
+                if fun():
+                    return True
 
         return False
 
     @return_false_on_nosuch
     def check_sp(self) -> bool:
         if self.get_stat_percent("sp") < self.statthreshold.sp[0]:
-            if any(
-                [
-                    self.use_item("Spirit Gem"),
-                    self.use_item("Spirit Potion"),
-                    self.use_item("Spirit Elixir"),
-                    self.use_item("Last Elixir"),
-                ]
-            ):
-                return True
+            for fun in [
+                partial(self.use_item, "Spirit Gem"),
+                partial(self.use_item, "Spirit Potion"),
+                partial(self.use_item, "Spirit Elixir"),
+                partial(self.use_item, "Last Elixir"),
+            ]:
+                if fun():
+                    return True
 
         if self.get_stat_percent("sp") < self.statthreshold.sp[1]:
-            if any(
-                [
-                    self.use_item("Spirit Gem"),
-                    self.use_item("Spirit Potion"),
-                ]
-            ):
-                return True
+            for fun in [
+                partial(self.use_item, "Spirit Gem"),
+                partial(self.use_item, "Spirit Potion"),
+            ]:
+                if fun():
+                    return True
 
         return False
 
     @return_false_on_nosuch
     def check_overcharge(self) -> bool:
-        if self.is_with_spirit_stance:
+        if self._buffmanager.has_buff("Spirit Stance"):
             # If Spirit Stance is active, check if Overcharge and SP are below thresholds
             if any(
                 [
@@ -266,7 +245,7 @@ class BattleDriver(HVDriver):
             [
                 self.get_stat_percent("overcharge") > self.statthreshold.overcharge[1],
                 self.get_stat_percent("sp") > self.statthreshold.sp[0],
-                not self.is_with_spirit_stance,
+                not self._buffmanager.has_buff("Spirit Stance"),
             ]
         ):
             return self.apply_buff("Spirit Stance")
@@ -284,101 +263,147 @@ class BattleDriver(HVDriver):
         )
 
         if continue_elements:
-            ElementActionManager(self).click_and_wait_log(continue_elements[0])
+            self.element_action_manager.click_and_wait_log(continue_elements[0])
             self._create_last_debuff_monster_id()
             return True
         else:
             return False
 
+    def debuff_monster(self, debuff: str, nums: list[int]) -> bool:
+        debuff_skill = MONSTER_DEBUFF_TO_CHARACTER_SKILL[debuff]
+        if debuff_skill in self.forbidden_skills:
+            return False
+
+        monster_ids_with_debuff = (
+            self._monsterstatusmanager.get_monster_ids_with_debuff(debuff)
+        ) + [self.last_debuff_monster_id[debuff]]
+        for num in nums:
+            if num not in monster_ids_with_debuff:
+                self.attack_monster_by_skill(
+                    num, MONSTER_DEBUFF_TO_CHARACTER_SKILL[debuff]
+                )
+                self.last_debuff_monster_id[debuff] = num
+                return True
+        return False
+
     def attack_monster(self, n: int) -> bool:
-        elements = self.driver.find_elements(
-            By.XPATH, '//div[@id="mkey_{n}"]'.format(n=n)
-        )
+        elements = self.driver.find_elements(By.ID, "mkey_{n}".format(n=n))
 
         if not elements:
             return False
 
-        ElementActionManager(self).click_and_wait_log(elements[0])
+        self.element_action_manager.click_and_wait_log(elements[0])
         return True
 
+    def attack_monster_by_skill(self, n: int, skill_name: str):
+        self.click_skill(skill_name, iswait=False)
+        self.attack_monster(n)
+
     def attack(self) -> bool:
-        # Check if Orbital Friendship Cannon can be used
-        if all(
-            [
-                self.with_ofc,
-                self.get_stat_percent("overcharge") > 220,
-                self.is_with_spirit_stance,
-                self.monster_alive_count >= self.statthreshold.countmonster[1],
+        base_monster_ids = [1, 3, 5, 7, 9, 2, 4, 6, 8, 0]
+
+        def monster_ids_starting_with(ids: list[int], n: int) -> list[int]:
+            """Returns a list of monster IDs starting with the given number."""
+            return ids[ids.index(n) :] + ids[: ids.index(n)]
+
+        def resort_monster_alive_ids(bmlist: list[int]) -> list[int]:
+            """Returns a list of monster IDs starting with the given number."""
+            monster_alive_ids = [
+                id
+                for id in bmlist
+                if id in self._monsterstatusmanager.alive_monster_ids
             ]
+            if len(self.system_monster_alive_ids):
+                monster_alive_ids = monster_ids_starting_with(
+                    monster_alive_ids, self.system_monster_alive_ids[0]
+                )
+            for monster_name in ["Yggdrasil", "Skuld", "Urd", "Verdandi"][::-1]:
+                monster_id = self._monsterstatusmanager.get_monster_id_by_name(
+                    monster_name
+                )
+                if monster_id in monster_alive_ids:
+                    monster_alive_ids = monster_ids_starting_with(
+                        monster_alive_ids, monster_id
+                    )
+            return monster_alive_ids
+
+        # Check if Orbital Friendship Cannon can be used
+        if (
+            self.with_ofc
+            and self.get_stat_percent("overcharge") > 220
+            and self._buffmanager.has_buff("Spirit Stance")
+            and self._monsterstatusmanager.alive_count
+            >= self.statthreshold.countmonster[1]
+            and self._skillmanager.get_skill_status("Orbital Friendship Cannon")
+            == "available"
         ):
-            self.click_skill("Orbital Friendship Cannon", iswait=False)
+            self.attack_monster_by_skill(
+                self._monsterstatusmanager.alive_monster_ids[0],
+                "Orbital Friendship Cannon",
+            )
+            return True
 
         # Get the list of alive monster IDs
-        monster_alive_ids = interleave_even_odd(self.monster_alive_ids)
-        if len(self.system_monster_alive_ids):
-            monster_id = self.system_monster_alive_ids[0]
-            monster_alive_ids = (
-                monster_alive_ids[monster_alive_ids.index(monster_id) :]
-                + monster_alive_ids[: monster_alive_ids.index(monster_id)]
-            )
-        for monster_name in ["Yggdrasil", "Skuld", "Urd", "Verdandi"][-1::-1]:
-            monster_id = self.get_monster_id_by_name(monster_name)
-            if monster_id in monster_alive_ids:
-                monster_alive_ids = (
-                    monster_alive_ids[monster_alive_ids.index(monster_id) :]
-                    + monster_alive_ids[: monster_alive_ids.index(monster_id)]
-                )
+        monster_alive_ids = resort_monster_alive_ids(base_monster_ids)
 
         # Get the list of monster IDs that are not debuffed with the specified debuffs
-        if all(
-            [
-                monster_alive_ids,
-                len(monster_alive_ids) > 3,
-                self.get_stat_percent("mp") > self.statthreshold.mp[1],
-            ]
+        if (
+            len(monster_alive_ids) > 3
+            and self.get_stat_percent("mp") > self.statthreshold.mp[1]
         ):
-            for debuff in ["Weaken", "Slow", "Blind", "MagNet", "Silence", "Drain"]:
-                if debuff in self.forbidden_skills:
+            for debuff in MONSTER_DEBUFF_TO_CHARACTER_SKILL:
+                if debuff in ["Imperiled"]:
                     continue
-                monster_with_debuff = (
-                    self._monsterstatusmanager.get_monster_ids_with_debuff(debuff)
-                )
-                if len(monster_with_debuff) / len(monster_alive_ids) < 0.7:
-                    for n in monster_alive_ids:
-                        if all(
-                            [
-                                n not in monster_with_debuff,
-                                n != self.last_debuff_monster_id[debuff],
-                            ]
-                        ):
-                            self.click_skill(debuff, iswait=False)
-                            self.attack_monster(n)
-                            self.last_debuff_monster_id[debuff] = n
-                            return True
+                if (
+                    len(monster_alive_ids)
+                    - len(
+                        self._monsterstatusmanager.get_monster_ids_with_debuff(debuff)
+                    )
+                    < 3
+                ):
+                    continue
+                if self.debuff_monster(debuff, monster_alive_ids):
+                    return True
 
         # Get the list of monster IDs that are not debuffed with Imperil
-        if self.get_stat_percent("mp") > self.statthreshold.mp[1]:
+        if (
+            "Imperil" not in self.forbidden_skills
+            and self.get_stat_percent("mp") > self.statthreshold.mp[1]
+        ):
             monster_with_imperil = (
-                self._monsterstatusmanager.get_monster_ids_with_debuff("Imperil")
+                self._monsterstatusmanager.get_monster_ids_with_debuff("Imperiled")
             )
         else:
             monster_with_imperil = monster_alive_ids
-        for n in monster_alive_ids:
-            if n not in monster_with_imperil:
-                if n == self.last_debuff_monster_id["Imperil"]:
-                    # If the last debuffed monster is the same, attack it directly
-                    if random() < 0.5:
-                        self.click_skill("Imperil", iswait=False)
-                    self.attack_monster(n)
-                else:
-                    self.click_skill("Imperil", iswait=False)
-                    self.attack_monster(n)
-                    self.last_debuff_monster_id["Imperil"] = n
+
+        n = monster_alive_ids[0]
+        if n in monster_with_imperil:
+            if self.get_stat_percent("overcharge") > 200 and self._buffmanager.has_buff(
+                "Spirit Stance"
+            ):
+                monster_health = self.battle_dashboard.monster_list[n].vitals.health
+                if self._skillmanager.get_skill_status("Vital Strike"):
+                    self.attack_monster_by_skill(n, "Vital Strike")
+                elif monster_health < 25 and self._skillmanager.get_skill_status(
+                    "Merciful Blow"
+                ):
+                    self.attack_monster_by_skill(n, "Merciful Blow")
             else:
-                self.last_debuff_monster_id["Imperil"] = -1
                 self.attack_monster(n)
-            return True
-        return False
+            self.last_debuff_monster_id["Imperiled"] = -1
+        else:
+            if n == self.last_debuff_monster_id["Imperiled"]:
+                # If the last debuffed monster is the same, attack it directly
+                if random() < 0.5:
+                    self.attack_monster_by_skill(n, "Imperil")
+                else:
+                    self.attack_monster(n)
+            else:
+                self.attack_monster_by_skill(
+                    n, MONSTER_DEBUFF_TO_CHARACTER_SKILL["Imperiled"]
+                )
+                self.last_debuff_monster_id["Imperiled"] = n
+        return True
 
     def finish_battle(self) -> bool:
         elements = self.driver.find_elements(
@@ -392,10 +417,7 @@ class BattleDriver(HVDriver):
         return True
 
     def use_channeling(self) -> bool:
-        channeling_elements = self.driver.find_elements(
-            By.XPATH, searchxpath_fun(["/y/e/channeling.png"])
-        )
-        if channeling_elements:
+        if "Channeling" in self.battle_dashboard.character.buffs:
             skill_names = ["Regen", "Heartseeker"]
             skill2remaining: dict[str, float] = dict()
             for skill_name in skill_names:
@@ -414,31 +436,30 @@ class BattleDriver(HVDriver):
 
     def battle_in_turn(self) -> str:
         self.turn += 1
+        self.clear_cache()
         # Print the current round logs
         print("\n".join(self.new_logs))
 
         if self.finish_battle():
             return "break"
 
-        if any(
-            fun()
-            for fun in [
-                self.go_next_floor,
-                PonyChart(self).check,
-                self.check_hp,
-                self.check_mp,
-                self.check_sp,
-                self.check_overcharge,
-                partial(self.apply_buff, "Health Draught"),
-                partial(self.apply_buff, "Mana Draught"),
-                partial(self.apply_buff, "Spirit Draught"),
-                partial(self.apply_buff, "Regen"),
-                partial(self.apply_buff, "Scroll of Life"),
-                partial(self.apply_buff, "Absorb"),
-                partial(self.apply_buff, "Heartseeker"),
-            ]
-        ):
-            return "continue"
+        for fun in [
+            self.go_next_floor,
+            PonyChart(self).check,
+            self.check_hp,
+            self.check_mp,
+            self.check_sp,
+            self.check_overcharge,
+            partial(self.apply_buff, "Health Draught"),
+            partial(self.apply_buff, "Mana Draught"),
+            partial(self.apply_buff, "Spirit Draught"),
+            partial(self.apply_buff, "Regen"),
+            partial(self.apply_buff, "Scroll of Life"),
+            partial(self.apply_buff, "Absorb"),
+            partial(self.apply_buff, "Heartseeker"),
+        ]:
+            if fun():
+                return "continue"
 
         if self.use_channeling():
             return "continue"
@@ -446,13 +467,19 @@ class BattleDriver(HVDriver):
         if self.attack():
             return "continue"
 
-        return "continue"
+        return "break"
 
     def _create_last_debuff_monster_id(self) -> None:
         self.last_debuff_monster_id: dict[str, int] = defaultdict(lambda: -1)
 
     def battle(self) -> None:
         self._create_last_debuff_monster_id()
+        partial_click_skill_menu = partial(
+            self.element_action_manager.click,
+            self.driver.find_element(By.ID, "ckey_skill"),
+        )
+        partial_click_skill_menu()
+        partial_click_skill_menu()
 
         while True:
             match self.pausecontroller.pauseable(self.battle_in_turn)():
