@@ -1,23 +1,45 @@
 from collections import defaultdict
 
 from selenium.webdriver.common.by import By
-from selenium.webdriver.remote.webdriver import WebDriver
+from selenium.webdriver.remote.webdriver import WebDriver, WebElement
 
-from .hv import HVDriver, searchxpath_fun
+from .hv import HVDriver
 from .hv_battle_action_manager import ElementActionManager
+from .hv_battle_dashboard import BattleDashBoard
 
 GEM_ITEMS = {"Mystic Gem", "Health Gem", "Mana Gem", "Spirit Gem"}
 
 
 class ItemProvider:
-    def __init__(self, driver: HVDriver) -> None:
+    def __init__(self, driver: HVDriver, battle_dashboard: BattleDashBoard) -> None:
         self.hvdriver: HVDriver = driver
+        self.battle_dashboard: BattleDashBoard = battle_dashboard
+        self.element_action_manager = ElementActionManager(
+            self.hvdriver, self.battle_dashboard
+        )
         self._checked_items: dict[str, str] = defaultdict(lambda: "available")
 
     @property
     def driver(self) -> WebDriver:
         return self.hvdriver.driver
 
+    @property
+    def items_menu_web_element(self) -> WebElement:
+        return self.hvdriver.driver.find_element(By.ID, "ckey_items")
+
+    def click_items_menu(self) -> None:
+        self.element_action_manager.click(self.items_menu_web_element)
+
+    def is_open_items_menu(self) -> bool:
+        """
+        Check if the items menu is open.
+        """
+        items_menum = self.items_menu_web_element.get_attribute("src") or ""
+        return "items_s.png" in items_menum
+
+    def get_pane_items(self) -> WebElement:
+        return self.hvdriver.driver.find_element(By.ID, "pane_item")
+
     def get_item_status(self, item: str) -> str:
         """
         回傳 'available', 'unavailable', 'not_found'
@@ -25,7 +47,9 @@ class ItemProvider:
         if self._checked_items[item] == "not_found":
             return "not_found"
 
-        item_divs = self.driver.find_elements(By.XPATH, f"//div/div[text()='{item}']")
+        item_divs = self.get_pane_items().find_elements(
+            By.XPATH, f"//div/div[text()='{item}']"
+        )
         if not item_divs:
             if item not in GEM_ITEMS:
                 self._checked_items[item] = "not_found"
@@ -37,6 +61,14 @@ class ItemProvider:
                 return "available"
         return "unavailable"
 
+    def get_item_elements(self, item: str) -> list[WebElement]:
+        return self.get_pane_items().find_elements(
+            By.XPATH,
+            "//div[@id and @onclick and div[@class='fc2 fal fcb']/div[text()='{item_name}']]".format(
+                item_name=item
+            ),
+        )
+
     def use(self, item: str) -> bool:
         if self._checked_items[item] == "not_found":
             return False
@@ -44,29 +76,12 @@ class ItemProvider:
         if self.get_item_status(item) == "unavailable":
             return False
 
-        if not self.driver.find_elements(
-            By.XPATH,
-            searchxpath_fun(["/y/battle/items_n.png", "/y/battle/items_s.png"]),
-        ):
-            return False
-
-        if not self.driver.find_elements(
-            By.XPATH, searchxpath_fun(["/y/battle/items_s.png"])
-        ):
-            item_menu_list = self.driver.find_elements(
-                By.XPATH, searchxpath_fun(["/y/battle/items_n.png"])
-            )
-            if not item_menu_list:
-                return False
-            ElementActionManager(self.hvdriver).click(item_menu_list[0])
-
-        item_button_list = self.driver.find_elements(
-            By.XPATH,
-            "//div[@id and @onclick and div[@class='fc2 fal fcb']/div[text()='{item_name}']]".format(
-                item_name=item
-            ),
-        )
+        item_button_list = self.get_item_elements(item)
         if not item_button_list:
             return False
-        ElementActionManager(self.hvdriver).click_and_wait_log(item_button_list[0])
+
+        if not self.is_open_items_menu():
+            self.click_items_menu()
+            item_button_list = self.get_item_elements(item)
+        self.element_action_manager.click_and_wait_log(item_button_list[0])
         return True
